/**************************************************************************
** 
** File name   : bstrapest.do
** Date        : April 23, 2003
** Author      : Branislav L. Slantchev (slantchev@ucsd.edu)
** Purpose     : Bootstrap the standard errors for the system of equations
** Requires    : 1perwar.dta generated by prepare.do
** Output      : bstrapest.dta with the bootstrapped estimates; this also
**               generates the frequency tables for observed/predicted
**               in scalar form; see the estimates of the o?p? variables
**
**************************************************************************/
version 8.0
#delimit ;

drop _all;
program drop _all;
set more off;

qui log;
loc logopen = r(status);
if "`logopen'" == "on" {; log close; };

set logtype text;
log using logs/bstrapest.log, replace;
di "$S_DATE $S_TIME";

use 1perwar, clear;

/*
** ALGORITHM
**    0. Resample data
**    1. Run duration model
**    2. Predict duration
**    3. Run outcome model using predicted duration
**    4. Save results (coeffs, variances)
**    5. Go back to 1, repeat `reps' times
**
** We call 'bootstrap' with our very own 'bs_accum' command, which does
** steps 1 through 3 (bootstrap takes care of the others).
*/

program define bs_accum, eclass;
    loc durmodel "parmil parpop terrain contig nstates tpop tmilper demo";
    loc outmodel "pwpop pwmil rloss salient talks demo demoloss";
qui {;    
    stset months;
    streg `durmodel', robust dist(loglog) time;
    predict pmonths, median time;
    oprobit outcome pmonths `outmodel', robust;
    tabpred, quiet; /* returns e(okpred), e(okpct), e(okerror), e(okfreq) */
};

    /* okfreq matrix is the two-way tabulation of observed-predicted values 
    ** since we have 4 outcomes, it's 4x4, or 16 entries; return them all
    ** to the bootstrap routine separately marked as o1p1, o1p2, etc.
    */
    mat freq = e(okfreq);
    
    ereturn scalar o1p1 = freq[1,1];
    ereturn scalar o1p2 = freq[1,2];
    ereturn scalar o1p3 = freq[1,3];
    ereturn scalar o1p4 = freq[1,4];

    ereturn scalar o2p1 = freq[2,1];
    ereturn scalar o2p2 = freq[2,2];
    ereturn scalar o2p3 = freq[2,3];
    ereturn scalar o2p4 = freq[2,4];

    ereturn scalar o3p1 = freq[3,1];
    ereturn scalar o3p2 = freq[3,2];
    ereturn scalar o3p3 = freq[3,3];
    ereturn scalar o3p4 = freq[3,4];

    ereturn scalar o4p1 = freq[4,1];
    ereturn scalar o4p2 = freq[4,2];
    ereturn scalar o4p3 = freq[4,3];
    ereturn scalar o4p4 = freq[4,4];
end;

/* set seed to 160773 to replicate the reported results, otherwise the
** bootstrap will produce different results because it's a random process
*/
set seed 160773;
loc reps = 1000;
bootstrap "bs_accum" _b okpred=e(okpred) okpct=e(okpct) okerror=e(okerror)
    o1p1=e(o1p1) o1p2=e(o1p2) o1p3=e(o1p3) o1p4=e(o1p4)
    o2p1=e(o2p1) o2p2=e(o2p2) o2p3=e(o2p3) o2p4=e(o2p4)
    o3p1=e(o3p1) o3p2=e(o3p2) o3p3=e(o3p3) o3p4=e(o3p4)
    o4p1=e(o4p1) o4p2=e(o4p2) o4p3=e(o4p3) o4p4=e(o4p4), 
    reps(`reps') dots saving(bstrapest) replace nonormal;

/*
** display the bootstrapped point estimates and confidence intervals;
** the mean is the expected value, the standard deviation is the standard error
** and (of course), the centiles give the confidence intervals.
*/
preserve;
qui use bstrapest, clear;
su;
centile, centile(5 95);
restore;

set more on;
log close;

exit;
