/**************************************************************************
** 
** File name   : duration.do
** Date        : May 28, 2003
** Author      : Branislav L. Slantchev (slantchev@ucsd.edu)
** Purpose     : Estimate the duration models
** Requires    : 1perwar.dta from prepare.do
** Output      : uncertainty.eps with effect of uncertainty on duration
**
**************************************************************************/
version 8.0
#delimit ;

drop _all;
set more off;

qui log;
loc logopen = r(status);
if "`logopen'" == "on" {; log close; };

set logtype text;
log using logs/duration.log, replace;
di "$S_DATE $S_TIME";

use 1perwar, clear;
stset months;

loc basemodel "parmil parpop";
loc terrmodel "`basemodel' terrain contig";
loc fullmodel "`terrmodel' nstates tpop tmilper demo";

/*
** check the proportional hazards assumption
*/
stcox `fullmodel', robust nolog sch(sch*) sca(sca*);
stphtest, rank detail;
qui drop sch* sca*;

/*
** our main variables exhibit no evidence of non-proportionality
** however, terrain and contig do seem to have it; we'll run a
** model that does not make the PH assumption (accelerated hazard)
*/

/*
** now let's check which distribution to use
** we get DF (expl. vars) + constant + ancillary param (if any)
*/
streg `fullmodel', robust dist(loglog) time nolog;
loc log_aic=2*(e(df_m)+3)-2*(e(ll));
loc log_ll=e(ll);

streg `fullmodel', robust dist(weibull) time nolog;
loc wbl_aic=2*(e(df_m)+3)-2*(e(ll));
loc wbl_ll=e(ll);

streg `fullmodel', robust dist(exponential) time nolog;
loc exp_aic=2*(e(df_m)+2)-2*(e(ll));
loc exp_ll=e(ll);

di "AIC(loglog)      = " `log_aic' " Log-likelihood: " `log_ll';
di "AIC(weibull)     = " `wbl_aic' " Log-likelihood: " `wbl_ll';
di "AIC(exponential) = " `exp_aic' " Log-likelihood: " `exp_ll';

/*
** we choose the loglog model based on both the AIC and the log-likelihoods
*/

/*
** estimate the basline model, nothing much to see here
*/
streg `basemodel', robust dist(loglog) time;

/*
** estimate the basline model, adding the territorial variables
*/
streg `terrmodel', robust dist(loglog) time;

/*
** estimate the full model, nothing much to see here
*/
streg `fullmodel', robust dist(loglog) time;
predict pmonths, median time;
gen p_error=pmonths-months;
gen p_abserr=abs(pmonths-months);
su months pmonths p_*;
centile months pmonths p_*, centile(50);

/*
** which wars did we really screw up predicting?
*/
qui su p_error;
list name months pmonths p_error if p_error == r(min) | p_error == r(max);

qui su parmil;
loc min_unc = r(min);
loc max_unc = r(max);

stcurve, survival 
    xlabel(0(12)108) xtitle("Duration of War in Months")
    ylabel(0(0.25)1) ytitle("Continuation Probability")
    at1(parmil=`min_unc' nstates=2)
    at2(parmil=`max_unc' nstates=2)
    legend(ring(0) position(2) cols(1))
    legend(label(1 "Least uncertainty (military preponderance)") 
           label(2 "Most uncertainty (military parity)"))
    clwidth(thick thick)
    clpattern(solid dash)
    title("");

graph export graphs/uncertainty.eps, replace as(eps) logo(off);
    
/*
** we are not saving the predicted duration because we shall be bootstrapping
** the estimations of the outcome model anyway
*/

set more on;
log close;
