/**************************************************************************
** 
** File name   : predict.do
** Date        : December 13, 2003
** Author      : Branislav L. Slantchev (slantchev@ucsd.edu)
** Purpose     : Generate predictions for specific wars
** Requires    : bstrapest.dta generated by bstrapest.do, and modified
**               versions of Clarify commands estsimp and sumqi (these
**               are included as estsimpx and simqix with this package)
** Description : Take the bootsrtrap estimations from bstrapest.dta
**               and run Monte Carlo simulations to plot the predicted
**               outcome probabilities.
**
**************************************************************************/
version 8.0
#delimit ;

program drop _all;

/*
** Prepare dataset for Monte Carlo simulations with simout
** args: num_obs to use for simulations
*/
program define prepout;
    args nobs;
    
    set obs `nobs';
    * generate sequence of num_obs duration values;
    qui su months;
    loc start = r(min);
    loc step  = `start' + (r(max)-r(min))/(_N-1);
    egen pmonths = fill(`start' `step');
    label var pmonths "War Duration (pmonths)";

    * generate placeholders for simulated expected values and CI;
    quiet {;
        gen epr1 =-1; gen epr2 =-1; gen epr3 =-1; gen epr4 =-1;
        gen cilo1=-1; gen cilo2=-1; gen cilo3=-1; gen cilo4=-1;
        gen cihi1=-1; gen cihi2=-1; gen cihi3=-1; gen cihi4=-1;
        label var epr1 "Pr(Defeat)";
        label var epr2 "Pr(Conces.)";
        label var epr3 "Pr(Gains)";
        label var epr4 "Pr(Victory)";
    };
end;

/*
** Monte Carlo simulations of the outcome of a particular war
** NOTE: call only after prepout is called (to generate placeholders)
*/
program define simwar;
    args fname thetitle warid;
    loc pctdone 0;
    loc step;
    loc ax 0;
    loc which 0;
    loc nwars 0;
    
    * step-size is 10%;
    loc step = _N/10;

    * get obs. number for this war... stupid, but works;
    qui {;
        sort warid;
        count if warid != .;
        loc nwars = r(N);
        forvalues i = 1/`nwars' {;
            if warid[`i'] == `warid' {;
                loc which = `i';
                continue, break;
            };
        };
    };

    * sanity check;
    if `which' == 0{;
        di in r "Error: no observation with warid=`warid'";
        exit;
    };

    loc war_name = name[`which'];

    * get the values of vars for warid;
    setx pwpop pwpop[`which'] pwmil pwmil[`which'] rloss rloss[`which']
         talks talks[`which'] salient salient[`which'] 
         demo demo[`which'] demoloss demoloss[`which'];

    set more off;
    di in gr "Simulating: `war_name'";
    di in gr "---------------------";
    di in gr "   pwpop = " in w %8.4f pwpop[`which'];
    di in gr "   pwmil = " in w %8.4f pwmil[`which'];
    di in gr "   rloss = " in w %8.4f rloss[`which'];
    di in gr " salient = " in w %8.4f salient[`which'];
    di in gr "   talks =   " in w cond(talks[`which'],"Yes","No");
    di in gr "    demo =   " in w cond(demo[`which'],"Yes","No");
    di in gr "demoloss = " in w %8.4f demoloss[`which'];
    di in gr "---------------------";
    di in gr "Working: " `pctdone' ".." _c;
  
    * loop for all duration values we want predictions for;
    loc k = 1;
    while `k' <= _N{;
        loc ax = `ax' + 1;
        if `ax' >= `step' {;
            loc ax = 0;
            loc pctdone = `pctdone' + 10;
            di in gr `pctdone' ".." _c;
        };

        * di "k = " `k' "; duration = " pmonths[`k'];
        * simulate outcome for this duration;
        qui setx pmonths pmonths[`k'];

        qui simqix, genpr(simpr1 simpr2 simpr3 simpr4);

        * save the expected probabilities and confidence intervals;
        loc i = 1;
        while `i' <= 4 {;
            loc sims "simpr`i'";
            loc pred "epr`i'";
            loc cilow "cilo`i'";
            loc cihigh "cihi`i'";
            
            su `sims', meanonly;
            qui replace `pred'=r(mean) if _n==`k';
            _pctile `sims', p(10 90);
            qui replace `cilow'  = _result(1) if _n==`k';
            qui replace `cihigh' = _result(2) if _n==`k';
            
            loc i = `i' + 1;
        };
        qui drop simpr*;
        loc k = `k' + 1;
    };
    set more on;
    di in gr "Done";
    
    line epr1 epr2 epr3 epr4 pmonths, sort
        clpattern(solid dash dot "_.")
        clwidth(thick thick thick thick)
        xlabel(0(12)108) xtitle("Duration of War in Months")
        ylabel(0(.25)1) ytitle("Outcome Probability")
        legend(ring(0) position(10) cols(1))
        legend(label(1 "Defeat") label(2 "Concessions") label(3 "Gains") label(4 "Victory"))
        title("`thetitle'", box bexpand);
        
    graph export graphs/`fname'.eps, replace as(eps) logo(off);
end;

/*
** This is the "fake" estimation command that will be called by the modified
** Clarify commands `estsimpx' and `simqix'; it behaves as if oprobit was
** called. The only modifications in the above two routines were the ones that
** ensure they recognize the 'fake' command as a valid estimation command.
**
** We should have the file 'bstrapest.dta' from the bootstrapping.
** It contains the 'correct' point estimates and standard errors.
** So, pretend as if we've run an ordered probit and gotten these results
** by running oprobit and then 'patching' the result matrices (this fills
** in all other parameters correctly).
*/
program define fake, eclass;
    version 8.0;
    /*
    ** the order of the variables must be the same as in the bootstrap
    ** estimation otherwise Clarify will complain about the estimation
    ** dropping variables.
    */
    loc rhsvars "pwpop pwmil rloss salient talks demo demoloss";
    
    qui oprobit outcome months `rhsvars', robust;
    loc rhsvars "pmonths `rhsvars'";

    mat b = e(b);
    loc nvars = colsof(b);
    mat ecat = e(cat);
    loc evcetype = e(vcetype);
    loc edepvar = e(depvar);
    loc ecmd = e(cmd);
    loc epredict = e(predict);
    sca ekcat = e(k_cat);
    sca en = e(N);
    
    qui bstat using bstrapest, nonormal nobc;
    /* use e(b) to get "observed" point estimates; the reason we are not
    ** using these to begin with is that there really are no "observed"
    ** estimates because there is no way to run the outcome models without
    ** some prediction for pmonths, which has its own error, etc.
    */
    mat bbig = e(bs_b);     
    mat Vbig = e(V);
    
    /* just get the coefficients and the covariance matrix for them */
    mat b = bbig[1,1..`nvars'];
    mat V = Vbig[1..`nvars',1..`nvars'];   
    
    /* we have to rename the row/column names to drop leading b_ */
    loc rnames : rownames(b);
    loc rnames = subinstr("`rnames'", "b_", "", .);
    mat rownames b = `rnames';

    loc cnames : colnames(b);
    loc nvars: word count `cnames';
    loc newcnames "";
    loc i = 1;
    while `i' <= `nvars' {;
        loc w: word `i' of `cnames';
        loc w = subinstr("`w'", "b_", "", 1);
        loc newcnames = "`newcnames' `w'";
        loc i = `i' + 1;
    };
    
    mat colnames b = `newcnames';

    loc cnames : colnames(V);
    loc nvars: word count `cnames';
    loc newcnames "";
    loc i = 1;
    while `i' <= `nvars' {;
        loc w: word `i' of `cnames';
        loc w = subinstr("`w'", "b_", "", 1);
        loc newcnames = "`newcnames' `w'";
        loc i = `i' + 1;
    };
    
    mat colnames V = `newcnames';
    mat rownames V = `newcnames';

    ereturn post b V;
    ereturn sca k_cat = ekcat;
    ereturn sca N = en;
    ereturn mat cat ecat;

    ereturn loc depvar `edepvar';
    ereturn loc vcetype `evcetype';
    ereturn loc cmd `ecmd';
    ereturn loc predict `epredict';
    ereturn loc rhsvars `rhsvars';

    glo S_E_cmd = "`ecmd'";
    glo S_E_depv = "`edepvar'";
    glo S_E_vce = "`evcetype'";
end;

/******************************************************************************/

drop _all;
set more off;

qui log;
loc logopen = r(status);
if "`logopen'" == "on" {; log close; };

set logtype text;
log using logs/predict.log, replace;
di "$S_DATE $S_TIME";
use 1perwar, clear;

loc nsims = 1000;
prepout `nsims';
estsimpx fake outcome pmonths pwpop pwmil rloss salient talks demo demoloss;

* args title fname warid;
simwar crimean "Crimean (Britain)" 47;
simwar anglo-egyptian "Anglo-Egyptian (Britain)" 94;
simwar lake-khasan "Lake Khasan (USSR)" 166;
simwar khalkhin-gol "Khalkhin Gol (Japan)" 167;
simwar yom-kippur "Yom Kippur (Egypt)" 208;
simwar iran-iraq "Iran-Iraq (Iraq)" 218;
simwar falklands "Falklands (Argentina)" 219;
simwar persian-gulf "First Persian Gulf (USA)" 222;
simwar second-boer "Second Boer (Boers)" 125;
simwar seven-weeks "Seven Weeks (Prussia)" 71;
simwar serbia-bulgaria "Serbo-Bulgarian (Serbia)" 100;
simwar six-days "Six Days (Israel)" 204;

set more on;
log close;

exit;
