/**************************************************************************
** 
** File name   : prepapre.do
** Date        : April 23, 2003
** Author      : Branislav L. Slantchev (slantchev@ucsd.edu)
** Purpose     : Prepares the data set used by all other programs
** Requires    : wars-1perwar.dta exported from Excel
** Output      : 1perwar.dta
**
**************************************************************************/
/*
** Prepare the wars-1perwar.dta (as exported from Excel) for use in the analysis.
*/
version 8.0
#delimit ;

drop _all;
set more off;

qui log;
loc logopen = r(status);
if "`logopen'" == "on" {; log close; };

set logtype text;
log using logs/prepare.log, replace;
di "$S_DATE $S_TIME";

use wars-1perwar, clear;

/* 
** scale losses to be in thousands (like population and military personnel)
*/
replace loss1=loss1/1000;
replace loss2=loss2/1000;

/*
** generate the ratio variables
*/
gen pwpop = pop1/(pop1+pop2);
label var pwpop "Pre-war balance of reserves A/(A+B)";
gen pwmil = milper1/(milper1+milper2);
label var pwmil "Pre-war military personnel ratio";
gen rloss = (loss1/milper1)/(loss1/milper1+loss2/milper2);
label var rloss "Ratio of losses per army size";

/*
** boolean and categorical variables 
*/
gen talks = io | nego;
label var talks "Pre-armistice talks/IO";
gen salient=cond(salient1>salient2,2,cond(salient1==salient2 & salient1,1,0));
label var salient "Issue salience for initiator";
label define labsal 0 "Not salient" 1 "Equally salient" 2 "More salient";
label val salient labsal;

/*
** generate several variables pertaining to initator
*/
gen demo = democ1 >= 6;
label var demo "Democratic initiator";
gen demoloss = demo*loss1;
label var demoloss "Initator Democracy*Losses";

gen months=duration/30;
label var months "War duration, in months";

/*
** uncertainty measures (close to parity: more)
*/
gen parmil = 1-abs(milper1-milper2)/(milper1+milper2);
gen parpop = 1-abs(pop1-pop2)/(pop1+pop2);
label var parmil "Military Parity (high values=more)";
label var parpop "Population Parity (high values=more)";

/*
** total measures
*/
gen tmilper = milper1+milper2;
label var tmilper "Total military personnel";
gen tpop = (pop1+pop2)/1000000;
label var tpop "Total population (/1000000)";

/*
** label remaining variables (be nice!)
*/
label define warlbl 1 "Defeat" 2 "Concessions" 3 "Gains" 4 "Victory";
label values outcome warlbl;  
label var outcome "War Outcome";
label var warid "War ID number";
label var name "War name";
label var yrbeg "Year war began";
label var side1 "Largest state on SideA (attacker)";
label var side2 "Largest state on SideB (target)";
label var nstates "Number of participants";
label var contig "Contiguity";
label var terrain "Terrain";

/*
** drop all variables we won't be using for the anlysis
*/
drop io nego salient1 salient2  cinc1 cinc2 pop1 pop2;
drop milper1 milper2 loss2 dist1 dist2  yrend;
drop xconst* democ1 democ2 autoc* duration loss1;

/*
** done: save the new dataset
*/
label data "Replication data set: 1 obs per war";
save 1perwar, replace;

set more on;
log close;
