#delimit ;
version 8.0;

/*
** Predict category, calculate % correct, and tabulate;
** call after oprobit
** syntax: tabpred varname [,quiet]
**
** there is a BUG: the program assumes that all observations
** in the dataset are used for the estimation (does not check
** for empty values after predict). this is a problem because
** it affects all calculations; drop all unnecessary observations
** before running this program (and oprobit itself)
*/

program define tabpred, eclass;
    syntax [if] [, QUIET];

    loc depvar = e(depvar);

    if e(cmd) ~= "oprobit" {;
        di in r "only oprobit supported";
        exit 139;
    };

    if "`depvar'" == "" {;
        di in r "last estimates not found";
        exit 139;
    };
    
    if "`if'" ~= "" {;
        loc if = "`if' & `depvar'~=."; 
    };
    else {;
        loc if = "if `depvar'~=.";
    };

    __tabpr `depvar' `if', `quiet';
end;

/********************************************[ internal programs ]********/
program define __tabpr, eclass;
    syntax varname if [, QUIET];
    tempvar freq;
    marksample touse;

* get the number of categories in the dependent variable;
    loc depvar "`varlist'";
    qui tab `depvar' if `touse';
    loc ncat = r(r);
    su `depvar' if `touse', meanonly;
    loc mincat = r(min);
    loc maxcat = r(max);

* build the list of variables for predict to generate;
    loc predvar "y0";
    loc i = 1;
    while `i' < `ncat' {;
        loc predvar "`predvar' y`i'";
        loc i = `i' + 1;
    };

* predict probs and drop empty observations;
    preserve;
    qui predict `predvar' if `touse', p;
    qui keep if y0~=.;

    qui summ `depvar' if `touse';
    loc nobs = r(N);

* calculate the frequency in the modal category;
    sort `depvar';
    qui by `depvar': gen double `freq' = sum(1) * `touse';
    qui by `depvar': replace `freq' = `freq'[_N];
    su `freq' if `touse', meanonly;
    qui count if `freq' == r(max);
    loc nmode = r(N);

* calculate total correct predictions from the model;
    qui egen ymax = rmax(`predvar');
    qui gen byte yhat = `mincat' if ymax==y0;
    loc i = 1;
    while `i' < `ncat' {;
        qui replace yhat = `mincat' + `i' if ymax==y`i';
        loc i = `i' + 1;
    };
    loc lab: variable label `depvar';
    loc lab "Predicted `lab'";
    label var yhat "`lab'";
    loc lab: value label `depvar';
    label value yhat `lab';

* display model, random, and modal correct predictions;
    di;
    di in gr "Observations predicted correctly";
    di in gr _dup(32) "-";
    di in gr _col(4) "Total obs  = " %5.0f `nobs';
    di in gr _col(4) "Categories = " %5.0f `ncat';
    di in gr _dup(32) "-";
* calculate random prediction (naive);
    loc okpct = 100/`ncat';
    loc okpred = `nobs'/`ncat';
    di in gr _col(4) "Random     = " in ye %5.0f `okpred'
        in gr " (" in ye %2.0f `okpct' in gr "%)";
* calculate modal prediction (best we can do w/o model);
    loc okpred = `nmode';
    loc okpct = 100*`nmode'/`nobs';
    di in gr _col(4) "Modal      = " in ye %5.0f `okpred'
        in gr " (" in ye %2.0f `okpct' in gr "%)";
* calculate model prediction (hopefully best);
    qui count if `depvar'==yhat & `touse';
    loc okpred = r(N);
    loc okpct = 100*`okpred'/`nobs';
    di in gr _col(4) "Model      = " in ye %5.0f `okpred'
        in gr " (" %2.0f in ye `okpct' in gr "%)";
* calculate the proportional reduction in error from modal;
    loc pre  = 1 - (`nobs'-`okpred')/(`nobs'-`nmode');
    di in gr _col(4) "PRE(Modal) = " in ye %5.3f `pre';
    di in gr _dup(32) "-";
* display tabulation, if requested;
    if "`quiet'" == "" {; tab `depvar' yhat if `touse'; };
    else di;

/*
** we may not be able to 'tabulate' for the frequencies because
** sometimes we may not predict any occurrences of one of the
** outcomes, and so 'tabulate' will be missing an entire
** column; instead, calculate the frequencies by hand
*/
    qui tab1 yhat if `touse';
    loc yhatcat = r(r);

    * we did not predict one (or more) of the outcomes, check which ones;
    if `yhatcat' ~= `ncat' {;
        /* create the matrix to hold the frequencies */
        qui tab `depvar' `depvar' if `touse', matcell(mfreq);
        mat Zero = J(rowsof(mfreq),1,0);    /* ncat x 1 zero vector */
    
        loc ccat = `mincat';
        loc mcol = 1; /* the current column (for yhat) in frequency matrix */
        while `ccat' <= `maxcat' {;
            qui tab1 yhat if `touse' & yhat == `ccat';
            if `r(r)' == 0 {;   /* this one is missing, set column to 0 */
                mat mfreq[1,`mcol'] = Zero;
            };
            else {; /* this category was predicted */
                loc icat = `mincat';
                loc mrow = 1;
                while `icat' <= `maxcat' {;
                    qui count if `touse' & yhat == `ccat' & `depvar' == `icat';
                    mat mfreq[`mrow',`mcol'] = r(N);
                    loc icat = `icat' + 1;
                    loc mrow = `mrow' + 1;
                };
            };
            loc ccat = `ccat' + 1;
            loc mcol = `mcol' + 1;  
        };
        /*        
        di "Patching...";
        tab `depvar' yhat if `touse';
        mat list mfreq;
        */
    };
    else {;     /* we're fine, all possible outcomes predicted */
        * save the frequencies observed/predicted;
        qui tab `depvar' yhat if `touse', matcell(mfreq);   
    };
    restore;
        
* return the prediction info;
    ereturn scalar okerror = `pre';
    ereturn scalar okpred = `okpred';
    ereturn scalar okpct = `okpct';
    ereturn matrix okfreq mfreq;
end;
